const utils = require("./Utils");
const fs = require("fs-extra");
const redis = require("redis");

class ListingsController {

  constructor() {

    this.cache_timeout = 900; // 15min timeout

  }

  getImages(listing_uuid) {

    return new Promise(

      async (resolve, reject) => {

        const listing = await utils.db.query("SELECT listing_code_region, listing_code_item FROM listings WHERE listing_uuid = ?", [listing_uuid]);

        if (listing.length == 0)
          return resolve([]);


        fs.readdir(`./data/uploads/${listing_uuid}/`)
          .then(files => {

            return resolve(files);

          })

          .catch(err => {

            return reject(err);

          })


      }

    )

  }

  removeMultiple(array) {

    return new Promise(

      (resolve, reject) => {

        var processes = [];
        array.forEach(item => {

          processes.push(this.remove(item));

        })

        console.log(processes);

        Promise.all(processes)
          .then(success => {

            return resolve(true);

          })

          .catch(err => {

            console.log(err);
            return reject(err)

          })

      }

    )

  }

  remove(uuid) {

    return new Promise(

      (resolve, reject) => {

        var deletedRow;

        this.fetch(uuid, true)
          .then(item => {

            if (item.length == 0)
              return resolve({ success: true, message: 'no_rows_deleted' });

            console.log(item);
            deletedRow = item[0];
            console.log(deletedRow.listing_code_item)
            return utils.db.query("DELETE FROM listings WHERE listing_uuid = ?", [uuid]);

          })

          .then(async (success) => {

            console.log(deletedRow);
            await utils.db.query("UPDATE listings SET listing_index = listing_index - 1 WHERE listing_index > ? AND listing_region_name = ?", [deletedRow.listing_index, deletedRow.listing_region_name]);
            await utils.db.query(`
              update listings set listing_code_item = listing_code_item - 1 WHERE listing_code_region = ? AND listing_code_item > ?
            `, [deletedRow.listing_code_region, deletedRow.listing_code_item]);
            return fs.remove(`./data/uploads/${uuid}`);

          })

          .then(done => {

            return resolve(true)

          })

          .catch(err => {

            console.log(err);
            return reject(err)

          })

      }

    )

  }

  search(query) {

    return new Promise(

      (resolve, reject) => {

        if (query.trim() == "")
          return resolve([])

        if (ListingsController.cache.hasOwnProperty(query)) {

          console.log(ListingsController.cache[query].timeout);

          if (ListingsController.cache[query].timeout <= new Date().getTime() / 1000)
            delete ListingsController.cache[query];
          else
            return resolve(ListingsController.cache[query]);

        }

        utils.db.query(`SELECT * FROM listings WHERE LOWER(listing_title) LIKE LOWER(?)`, [`%${query}%`])
          .then(rows => {

            console.log("found");
            if (rows.length == 0)
              return resolve([]);

            ListingsController.cache[query] = [];

            rows.forEach(row => {

              row.url = '/dashboard/listings/edit/' + row.listing_uuid
              ListingsController.cache[query].push({ item: row, timeout: parseInt((new Date().getTime() / 1000) + this.cache_timeout) })

            })

            return resolve(ListingsController.cache[query]);

          })

          .catch(err => {

            return reject(err);

          })

      }

    )

  }

  fetch(uuid, simple) {

    return new Promise(

      async (resolve, reject) => {

        if (!simple) {
          var sqlListing = (await utils.db.query("SELECT * FROM listings WHERE listing_uuid = ?", [uuid]));
          if (sqlListing.length == 0)
            return -1;

          var images = (await fs.readdir(`./data/uploads/${sqlListing[0].listing_uuid}`))
          images = images.map(image => { return `/uploads/${sqlListing[0].listing_uuid}/${image}` });
          sqlListing[0].images = images;
          sqlListing[0].code = `${sqlListing[0].listing_code_region}${sqlListing[0].listing_code_item}`;

          console.log(sqlListing);

          return resolve(sqlListing);

        } else {

          return resolve(await utils.db.query("SELECT * FROM listings WHERE listing_uuid = ?", [uuid]));

        }

      }

    )

  }

  pages() {

    return new Promise(

      async (resolve, reject) => {

        var total = parseInt((await utils.db.query("SELECT * FROM listings")).length);

        var o = { pages: [] }, currentPage = 1;
        for (var i = 0; i < total; i++) {

          if (i % 30 === 0) {
            o.pages.push({
              number: currentPage,
              url: '/dashboard/listings/all/' + currentPage
            })
            currentPage += 1;
          }

        }


        return resolve(o)

      }

    )

  }

  fetchLatest(from, no_desc) {

    return new Promise(

      async (resolve, reject) => {

        if (!from)
          return resolve(await utils.db.query("SELECT * FROM listings ORDER BY listing_creation_date DESC LIMIT 10"));

        var end = (from * 15) || 15;
        var start = end - 15;

        console.log(start, end);

        if (no_desc)
          return resolve(await utils.db.query("SELECT listing_title, listing_uuid FROM listings ORDER BY listing_creation_date DESC LIMIT ?,?", [start, end]))

        return resolve(await utils.db.query("SELECT * FROM listings ORDER BY listing_creation_date DESC LIMIT ?,?", [start, end]))

      }

    )

  }


}

ListingsController.cache = {}

module.exports = ListingsController;
