const MySQL = require("./MySQL");
const ObjectID = require("bson-objectid");
const ListingsController = require("./ListingsController");
const db = new MySQL();
const fs = require("fs-extra");

module.exports = {

  db,
  listings: null,
  stringifyIndex: function (number) {

    let r = number.toString();
    if (number < 10)
      r = `00${r}`
    if (number < 100 && number >= 10)
      r = `0${r}`

    return r;

  },
  mv: function (path, newPath) {

    return new Promise(

      (resolve, reject) => {

        fs.rename(path, newPath, (err) => {

          if (err) {
            reject(err);
            return;
          }

          resolve(true)

        })

      }

    )

  },
  config: JSON.parse(fs.readFileSync('./config.json')),
  isValidJSON: function (json) {
    try {
      JSON.parse(json);
    } catch (e) {
      return false;
    }

    return true;
  },
  checkParameters: function (type, params) {

    return function (req, res, next) {

      const bodyType = ((type == "GET") ? req.params : req.body);
      var missing = [];

      for (var i = 0; i < params.length; i++) {

        if (!bodyType[params[i]])
          missing.push(params[i])

      }

      if (missing.length > 0)
        return res.status(400).json({ success: false, error: `missing_parameters`, args: missing })

      next();

    }

  },
  getUser: async (userid) => {

    var user = await db.query(`
      SELECT * FROM users WHERE userid = ?
    `, [userid])

    if (user.length == 0)
      return null;

    return user[0];

  },

  deleteSession: (sessionid) => {

    return new Promise(

      (resolve, reject) => {

        db.query("DELETE FROM sessions WHERE sessionid = ?", [sessionid])
          .then(success => {

            return resolve(true)

          })

          .catch(err => {

            return reject(err);

          })

      }

    )

  },
  verifySession: (sessionid) => {

    return new Promise(

      async (resolve, reject) => {

        var session = (await db.query("SELECT * FROM sessions INNER JOIN users on users.userid = sessions.userid WHERE sessionid = ?", [sessionid]))[0], now = (new Date().getTime() / 1000);

        if (!session)
          return reject({ success: false, error: 'session_expired', status: 403 })

        if (new Date(ObjectID(session.sessionid).getTimestamp()).getTime() / 1000 < now) {

          await db.query("DELETE FROM sessions WHERE sessionid = ?", [sessionid]);

          return reject({ success: false, error: 'session_expired', status: 403 });

        } else {

          return resolve({ success: true, userid: session.userid, isGod: session.god })

        }

      }

    )

  },
  generateSession: function (userid) {

    return new Promise(

      (resolve, reject) => {

        var sessionid = ObjectID.generate(parseInt((new Date().getTime() / 1000) + 7200)); // 2hrs session

        db.query("INSERT INTO sessions VALUES(?,?)", [sessionid, userid])
          .then(success => {

            resolve(sessionid);

          })

          .catch(err => {

            reject({ success: false, error: 'fatal', status: 500 })

          })

      }

    )

  }

}
