const express = require("express");
const utils = require('../../Utils');
const router = express.Router();

router.use((req, res, next) => {

    res.setHeader('Access-Control-Allow-Origin', utils.config.url);
    res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');
    res.setHeader('Access-Control-Allow-Headers', 'X-Requested-With,content-type');
    res.setHeader('Access-Control-Allow-Credentials', true);

    var sessionid = req.cookies.sid;

    console.log(sessionid);

    if (sessionid === undefined) {

        if (req.originalUrl != '/dashboard/login' && req.originalUrl != '/login') {
            res.redirect('/dashboard/login');
            return;
        }

        next();

    }

    utils.verifySession(sessionid)
        .then(async (success) => {

            res.user = utils.getUser(success.userid);
            if (req.originalUrl == '/login' || req.originalUrl == '/dashboard/login') {
                res.redirect('/dashboard')
                return;
            }

            res.user = await utils.getUser(success.userid);
            res.userid = success.userid;

            next();


        })

        .catch((err) => {

            console.log(req.originalUrl != '/dashboard/login');

            if (req.originalUrl != '/login' && req.originalUrl != '/dashboard/login') {
                res.redirect('/dashboard/login');
                return;
            }
            else
                next();

        })

})

router.post('/logout', (req, res) => {

    const sessionid = req.cookies.sid;

    utils.deleteSession(sessionid)
        .then(success => {

            res.json({ success: true })

        })

        .catch(err => {

            res.json({ success: false, error: 'fatal' })

        })

})

router.get('/login', (req, res) => {

    res.render('dashboard/login', {

        title: 'Login'

    })

})

router.get('/', (req, res) => {

    res.render('dashboard/index', {

        title: 'Home',
        user: res.user

    })

})

router.get('/languages|lang', (req, res) => {

    res.render('dashboard/lang', {

        title: 'Lingue',
        user: res.user

    })

})

router.use('/listings', require('./listings'));

module.exports = router;
