const express = require("express");
const fs = require("fs-extra");
const router = express.Router();
const ListingsController = require("../../ListingsController");
const listings = new ListingsController();
const utils = require("../../Utils")

router.get('/', async (req, res) => {

  var latest = await listings.fetchLatest();
  latest.forEach(listing => {

    listing.listing_description = listing.listing_description.replace(/\n/g, "\\\\n").replace(/\r/g, "\\\\r").replace(/\t/g, "\\\\t");

  })

  const l = await utils.db.query("SELECT * FROM codes ORDER BY id")

  res.render('dashboard/listings', {

    title: 'Annunci',
    provinces: l,
    jprovinces: JSON.stringify(l),
    user: res.user,
    latest: JSON.stringify(latest)

  })

})

router.get('/province/:code/:page', async (req, res) => {

  let start = (req.params.page * 10) - 10, end = start + 10;

  let p = parseInt(Math.ceil(((await utils.db.query("SELECT * FROM listings WHERE listing_province = ?", [req.params.code])).length / 10)));
  let pages = [...Array(p).keys()].map(i => i + 1);

  const listings = (await utils.db.query(`
    SELECT * FROM listings
    LEFT JOIN codes ON 
    listings.listing_province = codes.code 
    WHERE listing_province = ? AND listings.listing_code_item > ? AND listings.listing_code_item <= ?
  `
    , [req.params.code.toUpperCase(), start, end]))

  const images = [];

  listings.forEach(listing => {

    images.push(fs.readdir(`${__dirname}/../../../data/uploads/${listing.listing_uuid}`))

  })

  let image_array = (await Promise.all(images)).map(image => { return image[0] || "" })

  listings.map((listing, index) => {

    listing.preview_image = `/uploads/${listing.listing_uuid}/${image_array[index]}` || ""
    return listing;

  })


  let subtitle = `Clicca sull'annuncio desiderato per più dettagli`, title;

  if (listings.length == 0) {
    subtitle = '', title = 'Nessun annuncio trovato...'
  } else title = `${listings[0].name}`

  res.render('dashboard/listings_all', {

    title,
    subtitle,
    listings,
    pages,
    root_url: `/dashboard/listings/province/${req.params.code}/`,
    current_page: req.params.page

  })

})

router.get('/edit/:uuid', async (req, res) => {

  const listing = (await listings.fetch(req.params.uuid))[0];

  if (listing == -1)
    return res.redirect('/dashboard/listings')

  res.render('dashboard/listings_edit', {

    title: 'Modifica annuncio',
    user: res.user,
    listing,
    provinces: await utils.db.query("SELECT * FROM codes ORDER BY id"),

  })

})

router.get('/all', async (req, res) => {

  res.render('dashboard/listings_all', {

    title: 'Tutti gli annunci',
    provinces: JSON.stringify(await utils.db.query("SELECT * FROM codes ORDER BY id"))

  })

})

module.exports = router;