const express = require("express");
const router = express.Router();
const { randomBytes } = require("crypto")
const xss = require("xss-filters");
const utils = require("../../Utils")
const safe = xss.inHTMLData;

function verify(req, res, next) {

    res.setHeader('Access-Control-Allow-Origin', utils.config.url);
    res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');
    res.setHeader('Access-Control-Allow-Headers', 'X-Requested-With,content-type');
    res.setHeader('Access-Control-Allow-Credentials', true);

    var sessionid = req.cookies.sid;

    console.log(sessionid);

    if (sessionid === undefined)
        return res.status(401).json({ success: false, error: 'unauthorized' })

    utils.verifySession(sessionid)
        .then(async (success) => {

            res.user = utils.getUser(success.userid);

            res.user = await utils.getUser(success.userid);
            res.userid = success.userid;

            next();

        })

        .catch((err) => {

            return res.status(401).json({ success: false, error: 'unauthorized' })

        })

}

router.get('/', (req, res) => {

    res.json({
        message: 'hello'
    })

})

router.post('/editTranslation', verify, utils.checkParameters("POST", ["tid", "translation"]), (req, res) => {

    const tid = req.body.tid,
        translation = safe(req.body.translation);

    utils.db.query(`

    UPDATE translations SET translation = ? WHERE translation_id = ?
    
    `, [translation, tid])

        .then(success => {

            res.json({ success: true });

        })

        .catch(err => {

            res.status(500).json({ success: false, error: 'fatal' })

        })

})

router.get('/translation/:translation_id', utils.checkParameters("GET", ["translation_id"]), (req, res) => {

    const tid = req.params.translation_id;

    utils.db.query("SELECT * FROM translations WHERE translation_id = ?", [tid])
        .then(success => {

            res.json(success);

        })

        .catch(err => {

            res.status(500).json({ success: false, error: 'fatal' })

        })

})

router.post('/removeTranslation', verify, utils.checkParameters("POST", ["tid"]), async (req, res) => {

    const tid = req.body.tid;

    utils.db.query("DELETE FROM translations WHERE translation_id = ?", [tid])
        .then(success => {

            res.json(true)

        })

        .catch(err => {

            res.status(500).json({ success: false, error: 'fatal' })

        })

})

router.post('/addTranslation', verify, utils.checkParameters("POST", ["lang_id", "listing_uuid", "translation"]), async (req, res) => {

    const lang_id = req.body.lang_id,
        listing_uuid = req.body.listing_uuid,
        translation = safe(req.body.translation),
        translation_id = randomBytes(8).toString('hex');

    let check = await utils.db.query("SELECT * FROM translations WHERE lang_id = ? AND listing_uuid = ?", [lang_id, listing_uuid])

    if (check.length > 0)
        return res.status(403).json({ success: false, message: 'already' });

    utils.db.query(`

        INSERT INTO translations VALUES(?,?,?,?)
    
    `, [listing_uuid, lang_id, translation, translation_id])
        .then(success => {

            res.json({ success: true })

        })

        .catch(err => {

            res.status(500).json({ success: false, error: 'false' })

        })

})

router.get('/translations/:listing_uuid', (req, res) => {

    const uuid = req.params.listing_uuid;

    utils.db.query(`
    
        SELECT languages.lang_name, languages.lang_code, translations.translation_id FROM translations INNER JOIN languages ON
        languages.id = translations.lang_id
        WHERE translations.listing_uuid = ?

    `, [uuid])

        .then(success => {

            res.json(success)

        })

        .catch(err => {

            res.status(500).json({ success: false, error: 'fatal' })

        })


})

router.get('/availableTranslations/:listing_uuid', verify, (req, res) => {

    const uuid = req.params.listing_uuid;

    utils.db.query(`
            select languages.lang_name, languages.id from (select languages.*, listings.listing_title from 
                listings 
                join translations
                on translations.listing_uuid = listings.listing_uuid 
                inner join languages on 
                translations.lang_id = languages.id 
                where listings.listing_uuid = ?) o
                RIGHT OUTER JOIN languages on
                o.id = languages.id
                WHERE o.listing_title IS NULL GROUP BY languages.lang_name
    `, [uuid])
        .then(langs => {

            res.json(langs)

        })
        .catch(err => {

            res.status(500).json({ success: false, error: 'fatal' })

        })

})

router.get('/list', verify, async (req, res) => {

    return res.json(

        await utils.db.query("SELECT * FROM languages ORDER BY lang_index DESC")

    )

})

router.post('/remove', verify, utils.checkParameters('POST', ['id']), (req, res) => {

    const lang_id = req.body.id;

    utils.db.query("DELETE FROM languages WHERE id = ?", [lang_id])
        .then(success => {

            return utils.db.query("DELETE FROM translations WHERE lang_id = ?", [lang_id])

        })

        .then(success => {

            console.log(success);

            res.json(true)

        })

        .catch(err => {

            res.status(500).json(false)

        })

})

router.post('/add', verify, utils.checkParameters('POST', ["language_name"]), (req, res) => {

    const lang_name = safe(req.body.language_name),
        lang_id = randomBytes(4).toString('hex');

    utils.db.query("INSERT INTO languages (id, lang_name) VALUES(?,?)", [lang_id, lang_name])
        .then(success => {

            res.json({
                id: lang_id, lang_name
            })

        })

        .catch(err => {

            res.status(500).json({ success: false, error: 'fatal' })

        })

})

module.exports = router;
