const express = require("express");
const xss = require("xss-filters");
const multer = require('multer');
const utils = require("../../Utils");
const random = require("randomstring");
const fs = require("fs-extra");
const uuidv1 = require("uuid/v1");

const storage = multer.diskStorage({
    destination: (req, file, callback) => {
        callback(null, `./data/uploads/tmp`)
    },
    filename: (req, file, callback) => {
        callback(null, file.originalname + random.generate('4'));
    }
})
const uploader = multer({ storage })
const router = express.Router();

router.get('/', (req, res) => {

    res.json({
        message: 'hello'
    })

})

router.get('/regions', (req, res) => {

    utils.db.query(`
        select codes.region_name from codes inner join listings on listings.listing_province = codes.code group by codes.region_name;
    `)
        .then(success => {

            res.json(success)

        })
        .catch(err => {

            res.status(500).json({ success: false, error: 'fatal' })

        })
})

router.get('/getProvinces/:region', async (req, res) => {

    let name = req.params.region;

    res.json(

        await (utils.db.query("SELECT name, code FROM codes WHERE region_name = ?", [name]))

    )

})

router.get('/getListings/:region/:page', async (req, res) => {

    let start = req.params.page - 1;
    let q, query = req.query.provinces,
        starting_price = req.query.starting_price,
        max_price = req.query.max_price;

    if (isNaN(starting_price))
        starting_price = 0

    if (isNaN(max_price))
        max_price = 1000000000000000000;

    if (isNaN(start))
        return res.status(400).json({ success: false, error: 'malformed request' })

    if (query) {

        let backup = query.split(",");
        q = query.split(",")
        q = q
            .map(item => {

                return utils.db.query(`
          select * from (select count(*) count, listing_price
          FROM codes
          INNER JOIN listings ON listings.listing_province = codes.code 
          WHERE codes.region_name = ? AND codes.code = ?
          ORDER BY listings.listing_creation_date DESC) c
        `, [req.params.region, item, starting_price, max_price])

            })

        let count, finals = [];

        try {
            count = await Promise.all(q);
            count.forEach(
                (item, i) => {

                    if (!item[0].count)
                        return null;

                    let total = item[0].count, results;
                    let starting_index = total - start * 10;

                    finals
                        .push(
                            utils.db.query(`
  
                select listings.listing_title, codes.name, listings.listing_code_item, listings.listing_code_region, listings.listing_uuid, listings.listing_price, listings.listing_location 
                FROM codes
                INNER JOIN listings ON listings.listing_province = codes.code 
                WHERE codes.region_name = ? AND listings.listing_code_item <= ? AND listings.listing_code_item > ? AND codes.code = ?
                AND listings.listing_price >= ? AND listings.listing_price <= ?
                ORDER BY listings.listing_creation_date DESC
  
                `, [req.params.region, starting_index, starting_index - 10, backup[i], starting_price, max_price])
                        )
                }
            )

            let t = await Promise.all(finals),
                bigImages = [];

            for (let i = 0; i < t.length; i++) {

                let images = [];

                for (let k = 0; k < t[i].length; k++) {

                    t[i][k].listing_price = t[i][k].listing_price.toLocaleString();
                    t[i][k].image = "";
                    images.push(utils.listings.getImages(t[i][k].listing_uuid))

                }

                bigImages.push(await Promise.all(images))

            }

            for (let i = 0; i < t.length; i++) {

                for (let k = 0; k < t[i].length; k++) {

                    t[i][k].preview_image = `/uploads/${t[i][k].listing_uuid}/${bigImages[i][k][0]}`;

                }

            }

            res.json(t)

        } catch (err) {
            console.log(err);

            return res.status(500).json({ success: false, error: 'fatal' })
        }

    } else {

        q = await (utils.db.query(`
      select * from (select count(*) count
      FROM codes
      INNER JOIN listings ON listings.listing_province = codes.code 
      WHERE codes.region_name = ?
      ORDER BY listings.listing_creation_date DESC) c
    `, [req.params.region, starting_price, max_price]
        ))

        console.log(q);

        let total = q[0].count, results;

        let starting_index = total - start * 10;

        console.log(starting_index, starting_index - 10);

        utils.db.query(`

    select listings.listing_title, codes.name, listings.listing_code_item, listings.listing_code_region, listings.listing_uuid, listings.listing_price, listings.listing_location 
    FROM codes
    INNER JOIN listings ON listings.listing_province = codes.code 
    WHERE codes.region_name = ? AND listings.listing_index <= ? AND listings.listing_index > ?
    AND listings.listing_price <= ? AND listings.listing_price >= ?
    ORDER BY listings.listing_creation_date DESC

  `, [req.params.region, starting_index, starting_index - 10, max_price, starting_price])

            .then(rows => {

                const images = [];
                results = rows;

                results.forEach(row => {

                    row.listing_price = row.listing_price.toLocaleString();
                    images.push(utils.listings.getImages(row.listing_uuid))

                })

                return Promise.all(images)

            })

            .then(files => {

                for (let i = 0; i < files.length; i++) {

                    results[i].preview_image = `/uploads/${results[i].listing_uuid}/${files[i][0]}`;

                }

                res.json(results)

            })
            .catch(err => {

                console.log(err);

                res.status(500).json({ success: false, error: 'fatal' })

            })

    }

})

router.get('/getRegions', async (req, res) => {

    res.json(await utils.db.query("SELECT codes.region_name as name FROM listings INNER JOIN codes ON codes.code = listings.listing_province GROUP BY codes.region_name"))

})

router.post('/remove', (req, res) => {

    const sessionid = req.cookies.sid,
        collection = req.body.collection;


    console.log("COLLECTION" + collection);


    utils.verifySession(sessionid)
        .then(success => {

            console.log(success);

            if (!success.isGod)
                return res.status(401).json({ success: false, message: 'no_permission' });

            return utils.listings.remove(collection)

        })

        .then(success => {

            return res.json({ success: true });

        })
        .catch(err => {

            console.log(err);
            res.status(err.status || 500).json(err);

        })

})

router.get('/basicSearch', /*checkIfGod,*/(req, res) => {

    utils.listings.search(req.query.q)
        .then(items => {

            res.json(items);

        })

        .catch(err => {

            console.log(err);
            res.json(err.status || 500).json(err);

        })

})

router.post('/removeImage', /*checkIfGod,*/(req, res) => {

    const sessionid = req.cookies.sid,
        item = req.body.item,
        uuid = req.body.uuid;

    utils.verifySession(sessionid)
        .then(success => {

            if (!success.isGod)
                return res.status(401).json({ success: false, message: 'no_permission' });

            return fs.remove(`./data/uploads/${uuid}/${item}`);

        })

        .then(success => {

            res.json({ success: true });

        })

        .catch(err => {

            if (!utils.isValidJSON(err))
                err = { success: false, error: 'Errore generico' };

            res.status(err.status || 500).json(err);

            throw err;

        })

})

router.post('/edit', uploader.array('attachments'), /*checkIfGod,*/(req, res) => {

    const title = safe(req.body.title),
        listing_uuid = safe(req.body.uuid),
        place = safe(req.body.place),
        type = safe(req.body.type),
        area = safe(req.body.area),
        description = safe(req.body.description),
        pool = safe(req.body.pool) || null,
        garage = safe(req.body.garage) || null,
        heating = safe(req.body.heating) || null,
        expenses = safe(req.body.expenses) || null,
        energetic_class = safe(req.body.energetic_class) || null,
        ipe = safe(req.body.ipe) || null,
        rooms = safe(req.body.rooms),
        bathrooms = safe(req.body.bathrooms) || null,
        listing_price = safe(req.body.listing_price),
        listing_service_tax = safe(req.body.listing_service_tax),
        listing_total_price = safe(req.body.listing_total_price),
        listing_code_region = safe(req.body.listing_code_region),
        sessionid = req.cookies.sid,
        percentage = (listing_price * (listing_service_tax / 100));

    var code;
    let split = listing_code_region.split("|");
    let province = split[1];

    utils.verifySession(sessionid)
        .then(async (success) => {

            if (!success.isGod)
                return res.status(401).json({ success: false, message: 'no_permission' });

            var current_prov = (await utils.db.query("SELECT listing_index, listing_province, listing_region_name, listing_code_item, listing_code_region FROM listings WHERE listing_uuid = ?", [listing_uuid]));
            let current_code = `${current_prov[0]['listing_code_region']}${current_prov[0]['listing_code_item']}`,
                region_name = (await utils.db.query("SELECT region_name FROM codes WHERE code = ?", province))[0].region_name,
                current_region_name = (await utils.db.query("SELECT region_name FROM codes WHERE code = ?", current_prov[0].listing_province))[0].region_name

            console.log(current_prov);

            if (current_prov[0]['listing_province'] !== province) {

                var max = (await utils.db.query("SELECT MAX(CAST(`listing_code_item` AS UNSIGNED)) AS max FROM listings WHERE listing_code_region = ?", [split[0]]))[0].max;
                var max_index = (await utils.db.query("SELECT MAX(CAST(`listing_index` AS UNSIGNED)) AS max FROM listings WHERE listing_region_name = ?", [region_name]))[0].max;

                if (region_name != current_prov[0]['listing_region_name']) {
                    if (max_index == null)
                        max_index = 1;
                    else
                        max_index += 1;
                } else {
                    max_index = current_prov[0]['listing_index'];
                }

                if (max == null)
                    max = 1;
                else
                    max += 1;

                console.log(max);

                if (max < 10)
                    max = "00" + max.toString();
                if (max >= 10 && max < 100)
                    max = "0" + max.toString();
                else
                    max = max.toString();

                try {

                    utils.db.query(`

        UPDATE listings SET 
        listing_index = ?,
        listing_region_name = ?,
        listing_province = ?,
        listing_code_region = ?,
        listing_code_item = ?,
        listing_location = ?,
        listing_description = ?,
        listing_type = ?,
        listing_price = ?,
        listing_service_tax = ?,
        listing_area = ?,
        listing_total_rooms = ?,
        listing_total_bathrooms = ?,
        listing_pool = ?,
        listing_park = ?,
        listing_ac = ?,
        listing_expenses = ?,
        listing_energetic_class = ?,
        listing_ipe = ?,
        listing_title = ?
        WHERE listing_uuid = ?

      `, [
                        max_index,
                        region_name,
                        province,
                        split[0],
                        max,
                        place,
                        description,
                        type,
                        listing_price,
                        listing_service_tax,
                        area,
                        rooms,
                        bathrooms,
                        pool,
                        garage,
                        heating,
                        expenses,
                        energetic_class,
                        ipe,
                        title,
                        listing_uuid

                    ])

                    await utils.db.query("UPDATE listings SET listing_code_item = listing_code_item - 1 WHERE listing_index > ? AND listing_region_name = ?", [current_prov[0].listing_index, current_region_name]);

                    if (region_name != current_region_name)
                        await utils.db.query("UPDATE listings SET listing_index = listing_index - 1 WHERE listing_index > ? AND listing_region_name = ?", [current_prov[0].listing_index, current_region_name]);

                } catch (err) {

                    console.log(JSON.stringify(err));

                    res.status(500).json({ success: false, error: 'fatal' })

                }

            } else {

                code = current_code;
                max = current_prov[0]['listing_code_item'];

                try {

                    utils.db.query(`

        UPDATE listings SET 
        listing_location = ?,
        listing_description = ?,
        listing_type = ?,
        listing_price = ?,
        listing_service_tax = ?,
        listing_area = ?,
        listing_total_rooms = ?,
        listing_total_bathrooms = ?,
        listing_pool = ?,
        listing_park = ?,
        listing_ac = ?,
        listing_expenses = ?,
        listing_energetic_class = ?,
        listing_ipe = ?,
        listing_title = ?
        WHERE listing_uuid = ?

      `, [

                        place,
                        description,
                        type,
                        listing_price,
                        listing_service_tax,
                        area,
                        rooms,
                        bathrooms,
                        pool,
                        garage,
                        heating,
                        expenses,
                        energetic_class,
                        ipe,
                        title,
                        listing_uuid

                    ])

                } catch (err) {

                    console.log(err);
                    res.status(500).json({ success: false, error: 'fatal' })

                }

            }

        })

        .then(async (success) => {

            let files = [];

            req.files.forEach(file => {

                console.log(file.filename);

                files.push(utils.mv(`./data/uploads/tmp/${file.filename}`, `./data/uploads/${listing_uuid}/${random.generate(4)}.jpg`))

            })

            await Promise.all(files)

            res.json({ success: true, listing: req.body });

        })

        .catch(err => {

            console.log(err);

            if (!utils.isValidJSON(err))
                err = { success: false, error: 'Errore generico' };

            res.status(err.status || 500).json(err);
            throw err;

        })



})

router.post('/add', uploader.array('attachments'), /*checkIfGod,*/ async (req, res) => {

    const title = safe(req.body.title),
        place = safe(req.body.place),
        type = safe(req.body.type),
        area = safe(req.body.area),
        description = safe(req.body.description),
        pool = safe(req.body.pool) || null,
        garage = safe(req.body.garage) || null,
        heating = safe(req.body.heating) || null,
        expenses = safe(req.body.expenses) || null,
        energetic_class = safe(req.body.energetic_class) || null,
        ipe = safe(req.body.ipe) || null,
        rooms = safe(req.body.rooms),
        bathrooms = safe(req.body.bathrooms) || null,
        listing_price = safe(req.body.listing_price),
        listing_service_tax = safe(req.body.listing_service_tax),
        listing_total_price = safe(req.body.listing_total_price),
        listing_code_region = safe(req.body.listing_code_region),
        sessionid = req.cookies.sid,
        uuid = uuidv1(),
        percentage = (listing_price * (listing_service_tax / 100));

    var code;
    let split = listing_code_region.split("|");
    let province = split[1],
        region_name = (await utils.db.query("SELECT region_name FROM codes WHERE code = ?", province))[0].region_name;

    utils.verifySession(sessionid)
        .then(async (success) => {

            if (!success.isGod)
                return res.status(401).json({ success: false, message: 'no_permission' });

            var max = (await utils.db.query("SELECT MAX(CAST(`listing_code_item` AS UNSIGNED)) AS max FROM listings WHERE listing_code_region = ?", [split[0]]))[0].max;
            var max_index = (await utils.db.query("SELECT MAX(CAST(`listing_index` AS UNSIGNED)) AS max FROM listings WHERE listing_region_name = ?", [region_name]))[0].max;
            if (max == null)
                max = 1;
            else
                max += 1;

            if (max_index == null)
                max_index = 1;
            else
                max_index += 1;

            if (max < 10)
                max = "00" + max.toString();
            if (max >= 10 && max < 100)
                max = "0" + max.toString();
            else
                max = max.toString();

            code = split[0].toString() + max;

            console.log(region_name);


            utils.db.query(`

        INSERT INTO listings VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,UNIX_TIMESTAMP(),?,?)

      `, [

                max_index,
                region_name,
                province,
                split[0],
                max,
                place,
                description,
                type,
                listing_price,
                listing_service_tax,
                area,
                rooms,
                bathrooms,
                pool,
                garage,
                heating,
                expenses,
                energetic_class,
                ipe,
                title,
                uuid

            ])

        })

        .then(async (success) => {

            var files = [];

            await fs.ensureDir(`./data/uploads/${uuid}`)

            req.files.forEach(file => {

                files.push(utils.mv(`./data/uploads/tmp/${file.filename}`, `./data/uploads/${uuid}/${random.generate(4)}.jpg`))

            })

            return Promise.all(files);

        })

        .then(success => {

            res.json({ success: true, listing: { listing_uuid: uuid, listing_title: title } });

        })

        .catch(err => {

            console.log(err);

            if (!utils.isValidJSON(err))
                err = { success: false, error: 'Errore generico' };

            res.status(err.status || 500).json(err);
            throw err;

        })


})

var safe = xss.inHTMLData;

module.exports = router;